unit RollingShutterCalc;

interface

uses system.Types, vcl.graphics, windows, system.Classes, system.sysUtils,
     clipBrd,
     regutil, math, compile, maths;

const
   maxPoints = 5;
   maxVecteurVideo = 1024;
   pasCouleur = 16;
   resolutionCibleMin = 32;
   resolutionCibleMax = 6*pasCouleur;
   resolutionAcceptee = 192;
   maxPixelCount = 32767;
   crTab = #9;

type
  TCorrectionRS = class
      private
          FRSValue : double;
          FNbre : integer;
          FNPoints : integer;
          FHauteur : integer;
          FangleX : double;
          FEchelle : double;
          FSigneY : double;
          Farrondi : double;
          FincertXY : integer;
          FwithOrigine : boolean;
          FnomFichierVideo : string;
          FCorrigePosition : boolean;
          procedure setEchelle(avalue : double);
          procedure setRSValue(avalue : double);
      public
      XYc : array[0..maxPoints,0..MaxVecteurVideo] of TpointF;
      XY : array[0..maxPoints,0..MaxVecteurVideo] of Tpoint;
      Temps : array[0..MaxVecteurVideo] of double;
      TempsRS : array[0..maxPoints,0..MaxVecteurVideo] of double;  // Rolling Shutter
      procedure CalculRollingShutter;
      procedure SauveData;
      Function valeurXc(j,i : integer) : double;
      Function valeurYc(j,i : integer) : double;
      Function valeurTc(j,i : integer) : double;
      Function valeurX(j,i : integer) : double;
      Function valeurY(j,i : integer) : double;
      Function valeurT(i : integer) : double;
      Function StrX(j,i : integer) : String;
      Function StrY(j,i : integer) : String;
      Function StrT(i : integer) : String;
      procedure Interpole(jdebut,jfin,j0,index : integer);
      property RSvalue : double write setRSValue;
      property Nbre : integer write FNbre;
      property NPoints : integer write FNPoints;
      property hauteur : integer write FHauteur;
      property angleX : double write FangleX;
      property echelle : double write setEchelle;
      property incertXY : integer write FIncertXY;
      property signeY : double write FsigneY;
      property withOrigine : boolean write FwithOrigine;
      property corrigePosition : boolean write FcorrigePosition;
      property nomFichierVideo : string write FnomFichierVideo;
      constructor create;
  end;
  tRGBTripleArray = array[0..maxPixelCount] of TRGBTriple;
  pRGBTripleArray = ^tRGBTripleArray;

var correctionRS : TCorrectionRS;

implementation

uses regmain, regdde;

const
   degreLissage = 2;
   ordre = degreLissage+1;
   puissMax = 2*degreLissage;

type
   vectorPuiss = array[0..puissMax] of double;

constructor TCorrectionRS.create;
begin
    FCorrigePosition := true;
    FsigneY := -1;
    FEchelle := 1;
end;

procedure TCorrectionRS.interpole(jdebut,jfin,j0,index : integer);
var puissX,puissY : vectorPuiss;
    Ax,Ay : matrice;
    CoeffX,coeffY : matriceLigne;

procedure initXY;
var j : integer;
    locX,locY,locT,t : double;
    i : integer;
begin
    for i := 0 to puissMax do begin
        puissX[i] := 0;
        puissY[i] := 0;
    end;
    for i := 1 to ordre do begin
        Ax[i,ordre+1] := 0;
        Ay[i,ordre+1] := 0;
    end;
    for j := jdebut to jfin do begin
        t := tempsRS[index,j];
        locT := t;
        for i := 1 to puissMax do begin
            puissX[i] := puissX[i]+locT;
            puissY[i] := puissY[i]+locT;
            locT := locT*t;
        end;
        locX := xy[index,j].x;
        locY := xy[index,j].y;
        for i := 1 to ordre do begin
            Ax[i,ordre+1] := Ax[i,ordre+1]+locX;
            Ay[i,ordre+1] := Ay[i,ordre+1]+locY;
            locX := locX*t;
            locY := locY*t;
        end;
        puissX[0] := puissX[0] + 1;
        puissY[0] := puissY[0] + 1;
    end;
    for i := 1 to ordre do
        for j := 1 to ordre do begin
            Ax[i,j] := PuissX[i+j-2];
            Ay[i,j] := PuissY[i+j-2];
        end;
end;

var t0 : double;
begin
    initXY;
    try
    maths.Resolution(Ax,ordre,Coeffx);
    maths.Resolution(Ay,ordre,Coeffy);
    t0 := tempsRS[1,j0];
    xyC[index,j0].x := coeffX[1]+t0*(coeffX[2]+coeffX[3]*t0);
    xyC[index,j0].y := coeffY[1]+t0*(coeffY[2]+coeffY[3]*t0);
    except
    end;
end; // Interpole

procedure TCorrectionRS.CalculRollingShutter;
var i, j: integer;
begin
     if FRSValue=0 then begin
        for i := 0 to pred(FNbre) do
            for j := 0 to FNPoints do begin
                xyC[j,i] := xy[j,i];
                tempsRS[j,i] := temps[i]-temps[0];
            end;
     end
     else begin
         for i := 0 to pred(FNbre) do
         for j := 0 to FNPoints do
             tempsRS[j,i] := (temps[i]-temps[0])+FRSValue*xy[j,i].y/Fhauteur; // 1000 ms
         for j := 0 to FNPoints do
         if j=1
         then begin // pour le 1, on garde le point et on change le temps
            for i := 0 to pred(FNbre) do
                xyC[1,i] := xy[1,i]; // passage Tpoint integer  TpointF single ?
         end
         else begin
             interpole(0,2,0,j);
             for i := 1 to (FNbre-2) do
                 interpole(i-1,i+1,i,j);
             interpole(FNbre-3,FNbre-1,FNbre-1,j);
         end;
     end;
end;

function TCorrectionRS.valeurXc(j,i : integer) : double;
begin
    if j=0
       then result := xyC[0,i].x*cos(FangleX)-xyC[0,i].y*sin(-FangleX)
       else result := (xyC[j,i].x-xyC[0,i].x)*cos(FangleX)-(xyC[j,i].y-xyC[0,i].y)*sin(-FangleX);
    result := result * FEchelle;
end;

function TCorrectionRS.valeurYc(j,i : integer) : double;
begin
    if j=0
       then result := xyC[0,i].x*sin(-FangleX)+xyC[0,i].y*cos(FangleX)
       else result := (xyC[j,i].x-xyC[0,i].x)*sin(-FangleX)+(xyC[j,i].y-xyC[0,i].y)*cos(FangleX);
    result := result * FEchelle * FsigneY;
end;

function TCorrectionRS.valeurTc(j,i : integer) : double;
begin
    result := (temps[i]-temps[0])+FRSValue*xy[j,i].y/Fhauteur;
end;

function TCorrectionRS.valeurX(j,i : integer) : double;
begin
    if j=0
       then result := xy[0,i].x*cos(FangleX)-xy[0,i].y*sin(-FangleX)
       else result := (xy[j,i].x-xy[0,i].x)*cos(FangleX)-(xy[j,i].y-xy[0,i].y)*sin(-FangleX);
    result := result * FEchelle;
end;

function TCorrectionRS.valeurY(j,i : integer) : double;
begin
    if j=0
       then result := xy[0,i].x*sin(-FangleX)+xy[0,i].y*cos(FangleX)
       else result := (xy[j,i].x-xy[0,i].x)*sin(-FangleX)+(xy[j,i].y-xy[0,i].y)*cos(FangleX);
    result := result * FEchelle * FsigneY;
end;

function TCorrectionRS.valeurT(i : integer) : double;
begin
    result := temps[i]-temps[0];
end;

Function TCorrectionRS.StrX(j,i : integer) : String;
begin
   result := floatToStrPoint(FArrondi*round(valeurX(j,i)/FArrondi))
end;

Function TCorrectionRS.StrY(j,i : integer) : String;
begin
   result := floatToStrPoint(FArrondi*round(valeurY(j,i)/FArrondi))
end;

Function TCorrectionRS.StrT(i : integer) : String;
begin
   result := floatToStrF(valeurT(i),ffFixed,5,3)
end;

procedure TCorrectionRS.setEchelle(avalue : double);
begin
       Fechelle := avalue;
       FArrondi := power(10,floor(log10(abs(FEchelle))));
end;

procedure TCorrectionRS.setRSValue(avalue : double);
begin
    FRSValue := aValue/1000; // donnees en ms
end;

procedure TcorrectionRS.SauveData;
var tampon : TstringList;
    j0 : integer;
    incertTemps,incertPos : string;

Procedure AjouteOptionGr;
var j : integer;
begin
     tampon.add(SymbReg2+intToStr(FNPoints)+' NOMX');
     if FNPoints=1
        then tampon.add('x')
        else for j := 1 to FNPoints do tampon.add('x'+intToStr(j));
     tampon.add(SymbReg2+intToStr(FNPoints)+' NOMY');
     if FNPoints=1
        then tampon.add('y')
        else for j := 1 to FNPoints do tampon.add('y'+intToStr(j));
     tampon.add(SymbReg2+'1 ORTHO');
     tampon.add('1');
     tampon.add(SymbReg2+'1 FICHIER');
     tampon.add(FnomFichierVideo);
end;

procedure SauveTemps;
var i,j : integer;
    ligne : String;
begin
       Ligne := '';
       for j := j0 to FNPoints do
           Ligne := Ligne+'t'+intToStr(j)+crTab+
                          'x'+intToStr(j)+crTab+
                          'y'+intToStr(j)+crTab;
       for j := j0 to FNPoints do
           Ligne := Ligne+'xC'+intToStr(j)+crTab+
                          'yC'+intToStr(j)+crTab;
       delete(ligne,length(ligne),1);
       tampon.add(Ligne);
       Ligne := '';
       for j := j0 to FNPoints do
           Ligne := Ligne+'s'+crTab+'m'+crTab+'m'+crTab;
       for j := j0 to FNPoints do
           Ligne := Ligne+'m'+intToStr(j)+crTab+
                          'm'+intToStr(j)+crTab;
       delete(ligne,length(ligne),1);
       tampon.add(ligne);
       Ligne := '';
       for j := j0 to FNPoints do
           Ligne := Ligne+'Temps corrig'+crTab+'Abscisse'+crTab+'Ordonne'+crTab;
       for j := j0 to FNPoints do
           Ligne := Ligne+'Abscisse corrige'+crTab+'Ordonne corrige'+crTab;
       delete(ligne,length(ligne),1);
       tampon.add(ligne);
       for i := 0 to pred(FNbre) do begin
           ligne := '';
           for j := j0 to FNPoints do
               ligne := ligne+
                  floatToStrPoint(valeurTc(j,i))+crTab+
                  floatToStrPoint(valeurX(j,i))+crTab+
                  floatToStrPoint(valeurY(j,i))+crTab;
           for j := j0 to FNPoints do
               ligne := ligne+
                  floatToStrPoint(valeurXC(j,i))+crTab+
                  floatToStrPoint(valeurYC(j,i))+crTab;
           delete(ligne,length(ligne),1);
           tampon.add(ligne);
        end;
        if FIncertXY>0 then begin
              tampon.Add(symbReg2+IntToStr(FNbre)+' INCERT');
              for i := 0 to pred(FNbre) do begin
                 ligne := '';
                 for j := j0 to FNPoints do
                    ligne := ligne+incertTemps+crTab+IncertPos+crTab+IncertPos+crTab;
                 for j := j0 to FNPoints do
                    ligne := ligne+incertPos+crTab+incertPos+crTab;
                 delete(ligne,length(ligne),1);
                 tampon.Add(ligne);
             end;
        end;
end; // SauveTemps

procedure SauvePosition;
var i,j : integer;
    ligne : String;
begin
       Ligne := 't';
       if FNpoints=1
        then begin
           if FWithOrigine then
              Ligne := Ligne+crTab+'xO'+crTab+'yO';
           Ligne := Ligne+crTab+'x'+crTab+'y';
        end
        else for j := j0 to FNPoints do
           Ligne := Ligne+crTab+'x'+intToStr(j)+
                          crTab+'y'+intToStr(j);
       tampon.add(Ligne);
       Ligne := 's';
       for j := j0 to FNPoints do
           Ligne := Ligne+crTab+'m'+crTab+'m';
       tampon.add(Ligne);
       Ligne := 'Temps';
       if FWithOrigine then
          Ligne := Ligne+crTab+'Abscisse origine/camra'+crTab+'Ordonne origine/camra';
       for j := 1 to FNPoints do
          Ligne := Ligne+crTab+'Abscisse'+crTab+'Ordonne';
       tampon.add(Ligne);
       for i := 0 to pred(FNbre) do begin
           ligne := floatToStrPoint(valeurTc(1,i));
           for j := j0 to FNpoints do
               ligne := ligne+
                  crTab+floatToStrPoint(valeurXc(j,i))+
                  crTab+floatToStrPoint(valeurYc(j,i));
           tampon.add(ligne);
        end;
        if FIncertXY>0 then begin
              tampon.Add(symbReg2+IntToStr(FNbre)+' INCERT');
              for i := 0 to pred(FNbre) do begin
                 ligne := incertTemps;
                 for j := 1 to FNPoints do
                    ligne := ligne+crTab+incertPos+crTab+incertPos;
                 tampon.Add(ligne);
             end;
        end;
end; // SauvePosition

begin // SauveData
       tampon := TstringList.create;
       tampon.add('');
       calculRollingShutter;
       if FwithOrigine
          then j0 := 0
          else j0 := 1;
       incertTemps := floatToStrPoint((temps[1]-temps[0])/100);
       incertPos := floatToStrPoint(FIncertXY*abs(FEchelle));
       if FcorrigePosition or ((FNPoints=1) and not FwithOrigine)
           then sauvePosition
           else sauveTemps;
       AjouteOptionGr;
       ClipBoard.asText := tampon.text;
       if pageCourante=0
         then with FregressiMain do begin
             grandeursOpen;
             grapheOpen;
             if FormDDE.ImportePressePapier then begin
                ModifConfigAcq := SaveConfigAcq;
                FocusAcquisitionBtn.visible := true;
             end
         end
         else FormDDE.AjoutePressePapier;
       tampon.free;
       modeAcquisition := acqVideo;
end; // SauveData

initialization
     correctionRS := TCorrectionRS.create;
finalization
     correctionRS.free;
end.
