unit Grapheu;

interface

uses windows, sysUtils, graphics, classes, printers,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs, math,
     system.math.vectors,
     maths, fft;

const
        maxPixel = 2048;
        MaxVecteurArduino = 4096;
        SymbReg : char = '';
        SymbReg2 : char = '&';
        IntSansSignif = -1;
        MaxVecteurOptique = 4096;
        CouleurFond = clWhite;
        CouleurCourbeOptique = clBlack;
        mondeX = 0;
        mondeY = 1;
        mondeYmax = 6;
        VitesseRS232 : array[0..4] of integer = (9600,19200,38400,57600,115200);

type
        IndiceVecteurOptique = -MaxVecteurOptique..MaxVecteurOptique;
        IndiceVecteurArduino = 0..MaxVecteurArduino;
        TvecteurOptique = array[indiceVecteurOptique] of double;
        TvecteurArduino = array[indiceVecteurArduino] of double;
        indiceAxes = mondeX..mondeYMax;
        TstyleDrag = (bsNone,bsMesure,bsOrigine,bsEchelle1,bsEchelle2,
                      bsChrono,bsChronoInit,bsAxeX,bsAxeY);
        TarrayPoint = array[TstyleDrag] of Tpoint;

 Tmonde = class
    mini,maxi : double;
    a,b : double; // entier = rel*a+b
    constructor create;
 end;

 const
    couleurArduino :  array[indiceAxes] of Tcolor =
        (clBlack,clBlue,ClRed,clBlack,clGreen,clPurple,clYellow);
 type

 Tgraphe = class
      Canvas : Tcanvas;
      mondeX : Tmonde;
      mondeY : Tmonde;
      debutG,finG,pasG : integer;
      dimPix,decalage : double;
      LimiteCourbe : Trect;
      ValeurY,vertY,rougeY,bleuY : TvecteurOptique;
      ValeurX : TvecteurOptique;
      wR,wG,wB,wL : boolean;
      Constructor Create;
      Procedure DrawG;
      Procedure WindowXY(xr,yr : double;var xi,yi : integer);
      Function getXmonde(xi : integer) : double;
      procedure TraceReticule(x : integer);
      procedure TraceEcart(x1,x2 : integer);
      destructor Destroy;override;
   end;

   TgrapheArduino = class
      Canvas : Tcanvas;
      monde : array[indiceAxes] of Tmonde;
      LimiteCourbe : Trect;
      Valeur,ValeurNew : array[indiceAxes] of TvecteurArduino; // temps, sortie CAN
      mondeMax : integer;
      echelleYfixee : boolean;
      NbrePoints,NbrePointsNew : integer;
      indexRoll : integer;
      Constructor Create;
      Procedure DrawG;
      Procedure WindowXY(xr,yr : double;var xi,yi : integer;m : integer);
      destructor Destroy;override;
   end;

   TgrapheSon = class
        Canvas : Tcanvas;
        mondeX : Tmonde;
        mondeY : Tmonde;
        debutS,finS,pasS : integer;
        LimiteCourbe : Trect;
        Son : TvecteurSon;
        Enveloppe : boolean;
        Constructor Create;
        Procedure DrawG;
        Procedure WindowXY(xr,yr : double;var xi,yi : integer);
        Procedure MondeXY(xi,yi : integer;var xr,yr : double);
        Function Xecran(xr : double) : integer;
        Function indice(xe : integer) : integer;
        destructor Destroy;override;
   end;

var magnet : integer;

function PointRel(P : TPoint;angle : single;larg,haut : integer) : Tpoint;
function Fleche(P : TPoint;angle : single) : TPolygon;

IMPLEMENTATION

function PointRel(P : TPoint;angle : single;larg,haut : integer) : Tpoint;
var dx,dy : integer;
begin
     result := P;
     dx := round(larg*cos(angle)+haut*sin(angle));
     dy := round(-larg*sin(angle)+haut*cos(angle));
     result.offset(dx,dy);
end;

function Fleche(P : TPoint;angle : single) : TPolygon;
begin
    setLength(result,3);
    result[0] := PointRel(P,angle,2*magnet,magnet);
    result[1] := P;
    result[2] := PointRel(P,angle,2*magnet,-magnet);
    (*
    Fill.Kind := TBrushKind.Solid;
    Fill.Color := claBlack;
    DrawPolygon(Points,1);
    FillPolygon(Points,1);
    Finalize(Points);
    *)
end;

constructor Tmonde.Create;
begin
    inherited create;
    A := 1;
    B := 0;
    Maxi := 1;
    Mini := 0;
end;

Procedure TgrapheSon.MondeXY(xi,yi : integer;var xr,yr : double);
begin
    with mondeX do xr := (xi-B)/A;
    with mondeY do yr := (yi-B)/A
end;

Procedure TgrapheSon.WindowXY(xr,yr : double;var xi,yi : integer);
begin
    with mondeX do xi := round(A*xr+B);
    with mondeY do yi := round(A*yr+B);
end;

Procedure TgrapheArduino.WindowXY(xr,yr : double;var xi,yi : integer;m : integer);
begin
    with monde[0] do xi := round(A*xr+B);
    with monde[m] do yi := round(A*yr+B);
end;

Function TgrapheSon.Xecran(xr : double) : integer;
begin
    with mondeX do result := round(A*(xr-debutS)+B)
end;

Function TgrapheSon.indice(xe : integer) : integer;
begin
    with mondeX do result := round(debutS+(xe-B)/A)
end;

procedure TgrapheSon.DrawG;

procedure ChercheMonde;
begin
    with mondeX,LimiteCourbe do begin
         A := (right-left)/(finS-debutS);
         Enveloppe := A<0.2;
         B := left;
     end;
     with mondeY,LimiteCourbe do begin
         A := (top-bottom)/2;  // donnes entre -1 et +1
         B := bottom+A;
     end;
end;// ChercheMonde

procedure TraceCourbe(valeur : TarraySon);
var x,y : integer;
    i : integer;
begin with LimiteCourbe,mondeY do begin
      windowXY(0,valeur[debutS],x,y);
      Canvas.moveTo(x,y);
      for i := debutS to finS do if ((debutS mod pasS)=0) then begin
          windowXY(i-debutS,valeur[i],x,y);
          Canvas.LineTo(x,y);
      end;
end end;// TraceCourbe

procedure TraceEnveloppe(valeur : TarraySon);
var x1,y1,x2,y2 : integer;
    i,j,pas,jdebut : integer;
    minSon,maxSon : double;
    val : double;
begin with LimiteCourbe,mondeY do begin
      pas := trunc((finS-debutS)/(right-left));
      if pas<1 then pas := 1;
      for i := left to (right-2) do begin
          jdebut := debutS+(i-left)*pas;
          minSon := valeur[jdebut];
          maxSon := minSon;
          for j := succ(jdebut) to pred(jdebut+pas) do begin
              val := valeur[j];
              if val>maxSon
                  then maxSon := val
                  else if val<minSon
                       then minSon := val;
          end;
          windowXY(jdebut,minSon,x1,y1);
          windowXY(jdebut,maxSon,x2,y2);
          Canvas.moveTo(i,y1);
          Canvas.lineTo(i,y2);
      end; // i
end end;// TraceEnveloppe

begin with canvas do begin
     brush.style := bsSolid;
     brush.color := clWhite;
     pen.mode := pmCopy;
     pen.width := 1;
     pen.color := clBlue;
     FillRect(LimiteCourbe);
     if debutS>=finS then exit;
     ChercheMonde;
     if enveloppe
        then traceEnveloppe(son.valeur)
        else traceCourbe(son.valeur);
     if Son.stereo  then begin
        pen.color := clRed;
        if enveloppe
           then traceEnveloppe(son.valeurBis)
           else traceCourbe(son.valeurBis);
      end;
end end; // TgrapheSon.DrawG

destructor TgrapheSon.destroy;
begin
    mondeX.free;
    mondeY.free;
end;

destructor TgrapheArduino.destroy;
var m : indiceAxes;
begin
    for m := mondeX to mondeYmax do
        monde[m].free;
end;

constructor TgrapheSon.Create;
begin
   inherited create;
   mondeX := Tmonde.create;
   mondeY := Tmonde.create;
   finS := 0;
   debutS := 0;
   pasS := 1;
end;

constructor TgrapheArduino.Create;
var m : indiceAxes;
begin
   inherited create;
   for m := mondeX to mondeYMax do
       monde[m] := Tmonde.create;
   mondeMax := mondeY;
   NbrePoints := 0;
   indexRoll := 0;
   echelleYfixee := false;
end;

Function Tgraphe.getXMonde(xi : integer) : double;
begin
     with mondeX do result := (xi-B)/A;
     result := dimPix*result+decalage;
end;

Procedure Tgraphe.WindowXY(xr,yr : double;var xi,yi : integer);
begin
        if isIncorrect(xr) or isIncorrect(yr)
           then begin
                xi := -1;
                yi := -1;
           end
           else begin
              with mondeX do xi := round(A*xr+B);
              with mondeY do yi := round(A*yr+B);
           end;
end;

procedure Tgraphe.DrawG;

procedure ChercheMonde;
begin
    with mondeX,LimiteCourbe do begin
         A := (right-left)/(Maxi-Mini);
         B := left-Mini*A;
     end;
     with LimiteCourbe,mondeY do begin
         A := (top-bottom)/(Maxi-Mini);
         B := bottom-Mini*A;
     end;
end;// ChercheMonde

procedure TraceCouleur(vect : TvecteurOptique;couleur : Tcolor);
var x,y : integer;
    i : integer;
begin with LimiteCourbe,mondeY do begin
      if finG<=debutG then exit;
      windowXY(valeurX[debutG],vect[debutG],x,y);
      Canvas.pen.color := couleur;
      Canvas.moveTo(x,y);
      for i := debutG to finG do begin
           windowXY(valeurX[i],vect[i],x,y);
           if (x>0) and (y>0) then Canvas.LineTo(x,y)
      end;
end end;// TraceCouleur

begin
     with canvas do begin
         brush.style := bsSolid;
         brush.color := couleurFond;
         pen.mode := pmCopy;
         pen.width := 1;
         FillRect(LimiteCourbe);
     end;
     if finG>maxVecteurOptique then finG := maxVecteurOptique;
     if debutG<-maxVecteurOptique then debutG := -maxVecteurOptique;
     ChercheMonde;
     if wL then TraceCouleur(valeurY,couleurCourbeOptique);
     if wR then TraceCouleur(rougeY,clRed);
     if wB then TraceCouleur(bleuY,clBlue);
     if wG then TraceCouleur(vertY,clGreen);
end; // tgraphe.drawG

procedure TgrapheArduino.DrawG;

Procedure GetMinMax(V : TvecteurArduino;var mini,maxi : reel);
var i : integer;
begin
     Mini := V[0];
     Maxi := V[0];
     for i := 1 to pred(NbrePoints) do
         if V[i]>Maxi
            then Maxi := V[i]
            else if V[i]<Mini
               then Mini := V[i]
end;

procedure ChercheMinMax;
var m : indiceAxes;
    marge,interval : double;
begin
    if not EchelleYfixee then
    for m := mondeX to mondeMax do with monde[m] do
        GetMinMax(valeur[m], mini, maxi);
    for m := mondeY to mondeMax do with monde[m] do begin
      marge := (maxi - mini)/25;
      if marge<1E-6 then marge := 1e-6;
      maxi := maxi + marge;
      mini := mini - marge;
      A := (LimiteCourbe.top-LimiteCourbe.bottom)/(Maxi-Mini);
      B := LimiteCourbe.bottom-Mini*A;
    end;
    with monde[mondeX] do begin
         interval := (maxi-mini)/nbrePoints;
         if nbrePointsNew=0 then
            maxi := mini+interval*puiss2Sup(nbrePoints);
         A := (LimiteCourbe.right-LimiteCourbe.left)/(Maxi-Mini);
         B := LimiteCourbe.left-Mini*A;
    end;
end;// ChercheMinMax

procedure TraceCouleur;
var x,y : integer;
    i : integer;
    m : indiceAxes;
begin
  for m := mondeY to mondeMax do begin
      windowXY(valeur[0,indexRoll],valeur[m,indexRoll],x,y,m);
      Canvas.pen.color := couleurArduino[m];
      Canvas.moveTo(x,y);
      for i := indexRoll+1 to pred(nbrePoints) do begin
           windowXY(valeur[0,i],valeur[m,i],x,y,m);
           Canvas.LineTo(x,y)
      end;
      for i := 0 to indexRoll-1 do begin
           windowXY(valeur[0,i],valeur[m,i],x,y,m);
           Canvas.LineTo(x,y)
      end;
  end;
end;// TraceCouleur

procedure TraceGris;

procedure ChercheMinMax1;
var marge : double;
begin with monde[1] do begin
        GetMinMax(valeurNew[1], mini, maxi);
        marge := (maxi - mini)/25;
        if marge<1E-6 then marge := 1e-6;
        maxi := maxi + marge;
        mini := mini - marge;
        A := (LimiteCourbe.top-LimiteCourbe.bottom)/(Maxi-Mini);
        B := LimiteCourbe.bottom-Mini*A;
end end;// ChercheMinMax1

var x,y : integer;
    i : integer;
begin
      Canvas.pen.color := clSilver;
      chercheMinMax1;
      windowXY(valeur[0,0],valeurNew[1,0],x,y,1);
      Canvas.moveTo(x,y);
      for i := 0 to pred(nbrePointsNew) do begin
           windowXY(valeur[0,i],valeurNew[1,i],x,y,1);
           Canvas.LineTo(x,y)
      end;
end;// TraceGris

begin
     if mondeMax>mondeYmax then mondeMax := mondeYmax;
     if mondeMax<0 then exit;
     with canvas do begin
         brush.style := bsSolid;
         brush.color := couleurFond;
         pen.mode := pmCopy;
         pen.width := 1;
         FillRect(LimiteCourbe);
     end;
     if nbrePoints<3 then exit;
     ChercheMinMax;
     TraceCouleur;
     if nbrePointsNew>3 then TraceGris;
end; // tgrapheArduino.drawG

destructor Tgraphe.destroy;
begin
    mondeX.free;
    mondeY.free;
    inherited
end;

constructor Tgraphe.Create;
begin
   inherited;
   mondeX := Tmonde.create;
   mondeY := Tmonde.create;
   finG := 0;
   debutG := 0;
   pasG := 1;
end;

procedure TGraphe.TraceReticule(x : integer);
begin with Canvas do begin
    Pen.Color := clBlack;
    Pen.mode  := pmNotXor;
    moveTo(X, 0);
    lineTo(X, limiteCourbe.bottom);
    Pen.mode := pmCopy;
end; end;

procedure TGraphe.TraceEcart(x1,x2 : integer);
var y0 : integer;
begin
    if x1<>x2 then with Canvas do begin
       Pen.mode  := pmNotXor;
       y0 := limiteCourbe.bottom div 2;
       Pen.color := clHighLight;
       Pen.width := 3;
       moveTo(x1, y0);
       lineTo(x2, y0);
       Pen.width := 1;
       Pen.Color := clBlack;
       moveTo(x1, 0);
       lineTo(x1, limiteCourbe.bottom);
       Pen.mode := pmCopy;
    end;
end;

initialization
   magnet := screen.Width div 256;
   if magnet<5 then magnet := 5;
end.

