unit arduinoOscillo;

interface

uses
  Winapi.Windows, Winapi.Messages,
  System.SysUtils, System.Variants, System.Classes, System.ImageList,
  system.DateUtils,
  Vcl.Graphics, Vcl.Grids, Vcl.ComCtrls, clipbrd, Vcl.ToolWin,  Vcl.ImgList,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls,
  Vcl.Samples.Spin, vcl.htmlHelpViewer,
  OOmisc, adport, lnswin32, awuser,
  registry, parser10, inifiles, math,
  regutil, arduinoGraphe, arduinoOscilloCfg, compile, maths, uniteker;

type
  tResolution = (r10bits,r12bits); // Due Zero 12 bits
  TArduinoOscilloForm = class(TForm)
    grid: TStringGrid;
    ImageList1: TImageList;
    ToolBar1: TToolBar;
    ToolsBtn: TToolButton;
    StartBtn: TToolButton;
    RegressiBtn: TToolButton;
    PaintBox: TPaintBox;
    ToolBar2: TToolBar;
    intervalle: TLabel;
    SingleCB: TCheckBox;
    TriggerTB: TTrackBar;
    DureeLabel: TLabel;
    FechList: TComboBox;
    SynchroCB: TComboBox;
    StopBtn: TToolButton;
    UneVoieRB: TRadioButton;
    DeuxVoiesRB: TRadioButton;
    ToolButton1: TToolButton;
    VoieSerie: TApdComPort;
    ProgressBar: TProgressBar;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure ToolsBtnClick(Sender: TObject);
    procedure RegressiBtnClick(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure StartBtnClick(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject);
    procedure FechSEClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SynchroLBClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TriggerTBExit(Sender: TObject);
    procedure UneVoieRBClick(Sender: TObject);
    procedure VoieSerieTriggerAvail(CP: TObject; Count: Word);
    procedure VoieSerieTriggerData(CP: TObject; TriggerHandle: Word);
  private
    chaineLue : ansiString;
    liste : TstringList;
    lectureEnCours,arretEnCours : boolean;
    crTrigger : word;
    graphe : TgrapheArduino;
    Parser : Tparser;
    NbrePointsCourant : integer;
    ValeurReal : array[0..2] of TvecteurSinc; // sortie CAN
    FechOK,NvoiesOK,synchroOK,seuilOK : boolean;
    uniteTemps : TUnite;
    NbrePointsArduino : integer;
    NbreVoies : integer;
    resolution : Tresolution;
    tZero : TdateTime;
    timeOutRs232 : integer;
    {$IFDEF Debug}
    fichierDebug : textFile;
    procedure ecritDebug(z: string);
    procedure initDebug;
    {$ENDIF}
    function valeurPeriode : integer;
    procedure relance;
    procedure Stop;
  public
  end;

var
  ArduinoOscilloForm: TArduinoOscilloForm;

implementation

{$R *.dfm}

uses regmain, regdde;

const
  MaxResol : array[Tresolution] of integer = (1023,4095); // 10 bits usuel ; 12 bits Due Zero
  PauseRS232 = 25;  // 5*waitSerial = 5 ms dans script Arduino

procedure TArduinoOscilloForm.StopBtnClick(Sender: TObject);
begin
   startBtn.Enabled := true;
   stopBtn.Enabled := false;
   stop;
end;

procedure TArduinoOscilloForm.SynchroLBClick(Sender: TObject);
begin
    synchroOK := false;
    if lectureEnCours
       then relance
       else triggerTB.Visible := synchroCB.ItemIndex>0;
end;

procedure TArduinoOscilloForm.StartBtnClick(Sender: TObject);
var m : indiceAxes;
begin
    nbrePointsCourant := 0;
    startBtn.Enabled := False;
    stopBtn.Enabled := true;
    if not FechOK then begin
       voieSerie.outPut := AnsiString('T'+intToStr(ValeurPeriode));
       sleep(PauseRS232);
       FechOK := true;
    end;
    if not NvoiesOK then begin
       if deuxVoiesRB.checked
          then begin
             voieSerie.outPut := AnsiString('C2');
             NbreVoies := 2;
          end
          else begin
             voieSerie.outPut := AnsiString('C1');
             NbreVoies := 1;
          end;
       sleep(PauseRS232);
       NVoiesOK := true;
    end;
    if not synchroOK then begin
       voieSerie.outPut := AnsiString('H'+intToStr(SynchroCB.itemIndex));
       sleep(PauseRS232);
       synchroOK := true;
    end;
    if not seuilOK and (synchroCB.ItemIndex<>0) then begin
       voieSerie.outPut := AnsiString('G'+intToStr(round(TriggerTB.max)-round(TriggerTB.position)));
       sleep(PauseRS232);
       seuilOK := true;
    end;
    triggerTB.Visible := synchroCB.ItemIndex>0;
    if crTrigger=0 then crTrigger := voieSerie.AddDataTrigger(crCR,true);
    timeOutRs232 := ceil(20*NbrePointsArduino*10/voieSerie.baud)+1; // 20 : 50000,1024,1024 cr lf   10 : 10 transitions , 2* : marge
    ProgressBar.max := NbrePointsArduino;
    ProgressBar.position := 0;
    voieSerie.Output := AnsiString('Start');
    lectureEnCours := true;
    for m := mondeY to mondeYmax do
        graphe.monde[m].maxiStr := MaxResol[resolution].toString;
    TriggerTB.Max := MaxResol[resolution];
end;

procedure TArduinoOscilloForm.ToolsBtnClick(Sender: TObject);
var duree : double;
    j : integer;
begin
     if crTrigger>0 then begin
        voieSerie.RemoveTrigger(crTrigger);
        crTrigger := 0;
     end;
     VoieSerie.Open := False;
     if ArduinoOscilloDlg=nil then Application.CreateForm(TArduinoOscilloDlg,ArduinoOscilloDlg);
     ArduinoOscilloDlg.sincCB.checked := graphe.avecSinc;
     ArduinoOscilloDlg.ordreSincSE.value := graphe.ordreSinc;
     ArduinoOscilloDlg.ResolutionCB.Checked := resolution=r12bits;
     for j := 0 to pred(ArduinoOscilloDlg.NbreRG.itemIndex) do
         if nbrePointsArduino=strToInt(ArduinoOscilloDlg.nbreRG.items[ArduinoOscilloDlg.nbreRG.ItemIndex]) then
            ArduinoOscilloDlg.nbreRG.ItemIndex := j;
     if ArduinoOscilloDlg.ShowModal=mrOK then begin
        if ArduinoOscilloDlg.ResolutionCB.Checked
           then resolution := r12bits
           else resolution := r10bits;
        graphe.avecSinc := ArduinoOscilloDlg.sincCB.checked;
        graphe.ordreSinc := ArduinoOscilloDlg.ordreSincSE.value;
        nbrePointsArduino := strToInt(ArduinoOscilloDlg.nbreRG.items[ArduinoOscilloDlg.nbreRG.ItemIndex]);
        duree := ValeurPeriode*NbrePointsArduino;
        dureeLabel.Caption := ' Dure : '+uniteTemps.formatValeurEtUnite(duree/1e6);
        graphe.monde[mondeX].maxiStr := FloatToStrF(duree,ffFixed,6,1);
     end;
     if arduinoOscilloDlg.Comports.items.Count>0 then begin
        VoieSerie.Open := True;
        crTrigger := voieSerie.addDataTrigger(crCR,true);
        startBtn.Enabled := true;
     end;
   // dsactivation de la voie srie a rinitialis Arduino
     FechOK := false;
     NVoiesOK := false;
     synchroOK := false;
     seuilOK := false;
end;

procedure TArduinoOscilloForm.FechSEClick(Sender: TObject);
var duree : double;
begin
    duree := valeurPeriode*NbrePointsArduino;
    dureeLabel.Caption := ' Dure : '+uniteTemps.formatValeurEtUnite(duree/1e6);
    graphe.monde[mondeX].maxiStr := FloatToStrF(duree,ffFixed,6,1);
    FechOK := false;
    if lectureEnCours then relance;
end;

procedure TArduinoOscilloForm.FormActivate(Sender: TObject);
begin
    if not voieSerie.Open and
       IsPortAvailable(VoieSerie.ComNumber) then begin
           voieSerie.Open := True;
           startBtn.Enabled := true;
           if crTrigger=0 then crTrigger := voieSerie.AddDataTrigger(crCR,true);
           FechOK := false;
           NVoiesOK := false;
           synchroOK := false;
           seuilOK := false;
       end;
end;

procedure TArduinoOscilloForm.FormClose(Sender: TObject; var Action: TCloseAction);
var Rini : TMemInifile;
    j,col : integer;
begin
     Rini := TMemIniFile.create(NomFichierIni);
     Rini.WriteInteger('Arduino','Com',voieSerie.comNumber);
     Rini.WriteInteger('Arduino','Baud',voieSerie.baud);
     Rini.WriteInteger('Arduino','NbreOscillo',NbrePointsArduino);
     Rini.WriteInteger('Arduino','Seuil',triggerTB.position);
     Rini.WriteInteger('Arduino','Synchro',synchroCB.itemIndex);
     Rini.WriteInteger('Arduino','Resolution',ord(resolution));
     Rini.WriteBool('Arduino','Monocoup',singleCB.checked);
     Rini.WriteBool('Arduino','Sinc',graphe.avecSinc);
     if DeuxVoiesRB.Checked
         then Rini.WriteInteger('Arduino','NbreVoies',2)
         else Rini.WriteInteger('Arduino','NbreVoies',1);
     Rini.WriteInteger('Arduino','OrdreSinc',graphe.ordreSinc);
     for j := 1 to 2 do begin
         col := j+1;
         if grid.cells[col,0]<>'' then
            Rini.writeString('Arduino','Nom'+intToStr(j),grid.cells[col,0]);
         if grid.cells[col,1]<>'' then
            Rini.writeString('Arduino','Unite'+intToStr(j),grid.cells[col,1]);
         if grid.cells[col,2]<>'' then
            Rini.writeString('Arduino','Convert'+intToStr(j),grid.cells[col,2]);
     end;
     Rini.writeInteger('Arduino','Fech',FechList.ItemIndex);
     Rini.updateFile;
     Rini.Free;
     if crTrigger>0 then begin
        voieSerie.RemoveTrigger(crTrigger);
        crTrigger := 0;
     end;
     VoieSerie.Open := false;
end;

procedure TArduinoOscilloForm.FormCreate(Sender: TObject);
const nomDefaut : array[1..2] of string  = ('X','Y');
      uniteDefaut : array[1..2] of string  = ('V','V');
      convertDefaut : array[1..2] of string  = ('V*5.0/1024','V*5.0/1024');
var j,NbreCom,col : integer;
    Rini : TMemIniFile;
    duree : double;
begin
// VoieSerie
     {$IFDEF Debug}
     voieSerie.Tracing := tlOn;
     voieSerie.Logging := tlOn;
     {$ELSE}
     voieSerie.Tracing := tlOff;
     voieSerie.Logging := tlOff;
     {$ENDIF}

// Cration
     uniteTemps := Tunite.Create;
     uniteTemps.nomUnite := 's';
     graphe := TgrapheArduino.create;
     liste := TstringList.Create;
     Parser := Tparser.create(nil);

// Inifile
     Rini := TMemIniFile.create(NomFichierIni);
     graphe.avecSinc := Rini.ReadBool('Arduino','Sinc',false);
     graphe.ordreSinc := Rini.ReadInteger('Arduino','OrdreSinc',2);
     VoieSerie.ComNumber := Rini.ReadInteger('Arduino','Com',3);
     VoieSerie.Baud := Rini.ReadInteger('Arduino','Baud',57600);
     NbrePointsArduino := Rini.ReadInteger('Arduino','NbreOscillo',300);
     TriggerTB.position := Rini.ReadInteger('Arduino','Seuil',512);
     synchroCB.itemIndex := Rini.ReadInteger('Arduino','Synchro',0);
     DeuxVoiesRB.Checked := Rini.ReadInteger('Arduino','NbreVoies',2)=2;
     if DeuxVoiesRB.Checked then NbreVoies := 2 else NbreVoies := 1;
     triggerTB.Visible := synchroCB.ItemIndex>0;
     singleCB.checked := Rini.ReadBool('Arduino','Monocoup',true);
     resolution := Tresolution(Rini.readInteger('Arduino','Resolution',0));
     if not IsPortAvailable(VoieSerie.ComNumber) then begin
        NbreCom := 0;
        for j := 1 to MaxComHandles do
        if IsPortAvailable(j) then begin
            inc(NbreCom);
            VoieSerie.ComNumber := j;
        end;
        if NbreCom=0 then showMessage('Pas de voie srie');
     end
     else NbreCom := 1; // au moins
     FechList.ItemIndex := Rini.readInteger('Arduino','Fech',5);
     for j := 1 to 2 do begin
         col := j+1;
         grid.Cells[col,0] := Rini.readString('Arduino','Nom'+intToStr(j),nomDefaut[j]);
         grid.Cells[col,1] := Rini.readString('Arduino','Unite'+intToStr(j),uniteDefaut[j]);
         grid.Cells[col,2] := Rini.readString('Arduino','Convert'+intToStr(j),convertDefaut[j]);
         graphe.monde[j].maxistr := '1024';
         graphe.monde[j].ministr := '0';
     end;
     Rini.Free;

     grid.Cells[0,0] := 'nom';
     grid.Cells[0,1] := 'unit';
     grid.Cells[0,2] := 'conversion';
     grid.Cells[1,0] := 't';
     grid.Cells[1,1] := 's';
     grid.Cells[1,2] := '';
     lectureEnCours := false;
     crTrigger := 0;
     if nbreCom>0 then begin
         try
         voieSerie.Open := True;
         startBtn.Enabled := true;
         crTrigger := voieSerie.AddDataTrigger(crCR,true);
         except
            ShowMessage('Problme voie srie');
         end;
     end;
     graphe.canvas := paintBox.canvas;
     graphe.isOscillo := true;
     graphe.nbrePoints := 0;
     graphe.monde[mondeX].miniStr := '0';
     graphe.monde[mondeX].nom := 't';
     graphe.monde[mondeX].unite := 's';
     if deuxVoiesRB.checked
          then graphe.mondeMax := mondeY+1
          else graphe.mondeMax := mondeY;
     duree := valeurPeriode*NbrePointsArduino;
     dureeLabel.Caption := 'Dure : '+uniteTemps.formatValeurEtUnite(duree/1e6);
     graphe.monde[mondeX].maxiStr := FloatToStrF(duree,ffFixed,6,1);

     FechOK := false;
     NVoiesOK := false;
     synchroOK := false;
     seuilOK := false;
     progressBar.Max := NbrePointsArduino;
     {$IFDEF Debug}
     initDebug;
     {$ENDIF}
end;

procedure TArduinoOscilloForm.FormDestroy(Sender: TObject);
begin
   parser.free;
   uniteTemps.Free;
end;

procedure TArduinoOscilloForm.FormShow(Sender: TObject);
begin
    if not voieSerie.Open and
       IsPortAvailable(VoieSerie.ComNumber) then begin
           voieSerie.Open := True;
           if crTrigger=0 then crTrigger := voieSerie.AddDataTrigger(crCR,true);
           startBtn.Enabled := true;
           FechOK := false;
           NVoiesOK := false;
           synchroOK := false;
           seuilOK := false;
       end;
end;

procedure TArduinoOscilloForm.PaintBoxPaint(Sender: TObject);
begin
      graphe.drawG;
end;

procedure TArduinoOscilloForm.RegressiBtnClick(Sender: TObject);
var tampon : TstringList;
    i,imax : integer;
    Ligne : String;
    m : indiceAxes;

    procedure affecteValeur;
    var m : indiceAxes;
        i : integer;
    begin
     imax := pred(graphe.nbrePoints);
     for m := mondeY to graphe.mondeMax do
         if grid.cells[m+1,2]<>'' then begin
            Parser.expression := grid.cells[m+1,2];
            for i := 0 to imax do begin // conversion
                try
                   Parser.V := graphe.monde[m].valeur[i];
                   valeurReal[m,i] :=  Parser.Value;
                except
                   valeurReal[m,i] := graphe.monde[m].valeur[i];
                end;
            end;
         end
         else for i := 0 to imax do
             valeurReal[m,i] := graphe.monde[m].valeur[i];
     for i := 0 to imax do // data
          valeurReal[0,i] := graphe.monde[0].valeur[i]/1e6;
    end;

    procedure affecteSinc;
    var m : indiceAxes;
        i : integer;
    begin
     imax := pred(graphe.nbreSinc);
     for m := mondeY to graphe.mondeMax do
         if grid.cells[m+1,2]<>'' then begin
            Parser.expression := grid.cells[m+1,2];
            for i := 0 to imax do begin // conversion
                try
                   Parser.V := graphe.monde[m].valeurSinc[i];
                   valeurReal[m,i] :=  Parser.Value;
                except
                   valeurReal[m,i] := graphe.monde[m].valeurSinc[i];
                end;
            end;
         end
         else for i := 0 to imax do
             valeurReal[m,i] := graphe.monde[m].valeurSinc[i];
     for i := 0 to imax do // data
         valeurReal[0,i] := graphe.monde[0].valeurSinc[i]/1e6;
    end;

begin
     startBtn.Enabled := true;
     stopBtn.Enabled := false;
     stop;
     tampon := TstringList.create;
     tampon.add(Application.exeName);
     tampon.add('');
     ligne := '';
     for m := 0 to graphe.mondeMax do begin
         if grid.Cells[m+1,0]='' then grid.Cells[m+1,0] := 'x'+intToStr(m);
         Ligne := ligne+grid.Cells[m+1,0]+crTab;
     end;
     tampon.add(Ligne);  // noms
     ligne := '';
     for m := 0 to graphe.mondeMax do
         Ligne := ligne+grid.Cells[m+1,1]+crTab;  // units
     tampon.add(ligne);
     if graphe.avecSinc then affecteSinc else affecteValeur;
     for i := 0 to imax do begin // data
         ligne := '';
         for m := mondeX to graphe.mondeMax do
             Ligne := ligne+FloatToStrPoint(valeurReal[m,i])+crTab;
         tampon.add(Ligne);
     end;
     ClipBoard.asText := tampon.Text;
     if pageCourante=0
         then with FregressiMain do begin
             grandeursOpen;
             grapheOpen;
             if FormDDE.ImportePressePapier then begin
                ModifConfigAcq := SaveConfigAcq;
                FocusAcquisitionBtn.visible := true;
             end
         end
         else FormDDE.AjoutePressePapier;
     tampon.free;
     modeAcquisition := acqOscilloArduino;
     graphe.nbrePoints := 0;
     Close;
end; // RegressiBtnClick

procedure TArduinoOscilloForm.TriggerTBExit(Sender: TObject);
begin
    seuilOK := false;
    if lectureEnCours then relance
end;

procedure TArduinoOscilloForm.UneVoieRBClick(Sender: TObject);
begin
     if deuxVoiesRB.checked
       then graphe.mondeMax := mondeY+1
       else graphe.mondeMax := mondeY;
     NVoiesOK := false;
     if lectureEnCours then relance;
end;

function TArduinoOscilloForm.valeurPeriode : integer;
var chaine,intStr : string;
    i : integer;
    coeff : integer;
    carac : char;
begin
   chaine := FechList.text;
   i := 1;
   intStr := '';
   coeff := 1;
   while i<=length(chaine) do begin
       carac := chaine[i];
       if charInSet(carac,['0'..'9'])
          then intStr := intStr+carac
          else if carac='k' then coeff := 1000;
       inc(i);
   end;
   result := strToInt(intStr)*coeff;
   result := round(1e6/result);
end;

procedure TArduinoOscilloForm.VoieSerieTriggerAvail(CP: TObject; Count: Word);
var
   i: word;
   carac: ansiChar;
begin // TriggerAvail
   for i := 1 to Count do begin
      if voieSerie.CharReady then begin
         carac := voieSerie.GetChar;
         if (carac >= ' ') then chaineLue := chaineLue + carac;
      end;
    end
end;

procedure TArduinoOscilloForm.VoieSerieTriggerData(CP: TObject;
  TriggerHandle: Word);
var i : integer;
    m : indiceAxes;
    timeOut : boolean;
begin
      if not lectureEnCours then begin
         chaineLue := '';
         exit;
      end;
      try
      liste.commaText := string(chaineLue);
      chaineLue := '';
      if liste.Count<>(NbreVoies+1) then exit;
      for i := 0 to liste.Count-1 do
          graphe.monde[i].valeurNew[nbrePointsCourant] := strToFloatWin(liste[i]);
      except
      end;
      if NbrePointsCourant=0
      then begin
         tZero := getTime;
         timeOut := false;
      end
      else begin
         timeOut := milliSecondsBetween(getTime,tZero)>2*1000*timeOutRs232; // 1000 ms 2* marge
      end;
      inc(nbrePointsCourant);
      if ProgressBar.position < (nbrePointsCourant+16) then begin
         ProgressBar.position := nbrePointsCourant;
         ProgressBar.update;
      end;
      if (nbrePointsCourant>=NbrePointsArduino) or timeOut then
      if arretEnCours
      then arretEnCours := false
      else begin
            regressiBtn.Enabled := true;
            ProgressBar.position := nbrePointsCourant;
            if timeOut
               then graphe.nbrePoints := nbrePointsCourant
               else graphe.nbrePoints := nbrePointsArduino;
            for i := 0 to pred(nbrePointsArduino) do
                graphe.monde[0].valeur[i] := graphe.monde[0].valeurNew[i];
            for m := mondeY to graphe.mondeMax do
                for i := 0 to pred(nbrePointsArduino) do
                    graphe.monde[m].valeur[i] := graphe.monde[m].valeurNew[i];
            paintBox.Invalidate;
            if singleCB.Checked
               then begin
                  lectureEnCours := false;
                  startBtn.Enabled := True;
                  stopBtn.Enabled := false;
               end
               else begin
                  nbrePointsCourant := 0;
                  voieSerie.Output := AnsiString('Start');
                  // Arduino s'arrte aprs chaque lecture => Start ncessaire
                  lectureEnCours := true;
                  ProgressBar.position := 0;
                  ProgressBar.update;
               end;
      end;
end;

procedure TArduinoOscilloForm.relance;
begin
       stop;
       startBtnClick(nil);
end;

procedure TArduinoOscilloForm.Stop;
var Timer : EventTimer;
begin
        cursor := crHourGlass;
        arretEnCours := lectureEnCours and ((NbrePointsCourant>0) or (chaineLue<>''));
        voieSerie.Output := AnsiString('Stop');
        newTimerSecs(timer,timeOutRs232);
        ToolBar2.Enabled := false;
        ToolBar1.Enabled := false;
        repeat
            application.ProcessMessages;
        until not arretEnCours or timerExpired(timer);
        ToolBar2.Enabled := true;
        ToolBar1.Enabled := true;
        arretEnCours := false;
        lectureEnCours := false;
        cursor := crDefault;
end;

{$IFDEF Debug}
procedure TArduinoOscilloForm.ecritDebug(z: string);
begin
   append(fichierDebug);
   writeln(fichierDebug,z);
   closeFile(fichierDebug);
end;

procedure TArduinoOscilloForm.initDebug;
begin
     AssignFile(fichierDebug,extractFilePath(application.ExeName)+'oscillo.txt');
     Rewrite(fichierDebug);
     writeln(fichierDebug,'oscillo');
     closeFile(fichierDebug);
end;
{$ENDIF}

end.
